#========================================================================================
# package      DE_EPAGES::Klarna::t::Recalculate_Internal
# state        public
#----------------------------------------------------------------------------------------
# description  For Klarna Invoice fee
#========================================================================================
package DE_EPAGES::Klarna::t::Recalculate_Internal;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';
Readonly my $SHOPPATH => $ROOTPATH.'/TestShopKlarna';
Readonly my $ORDERPATH => $SHOPPATH.'/Customers/TestCustomer/Orders/1234-5678';
Readonly my $ORDERPATH_TAX_GROSS => $SHOPPATH.'/Customers/TestCustomer/Orders/1234-5679';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # Test
            eval {
                ok( Init(), 'initialization' );
                Test_InvoiceFee();
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub Test_InvoiceFee {
    my $subTestName = (caller(0))[3];

    my $Order = LoadObjectByPath($ORDERPATH_TAX_GROSS);


    my $PaymentTaxRateBefore = $Order->get('LineItemContainer')->get('Payment')->get('TaxRate');
    is($PaymentTaxRateBefore, 0.19, "$subTestName - Klarna TaxRate is correctly stored in LineItemPayment");

    my $PaymentLineItemPriceBefore = $Order->get('LineItemContainer')->get('Payment')->get('LineItemPrice');
    is($PaymentLineItemPriceBefore, 0, "$subTestName - Klarna LineItemPrice is zero in LineItemPayment");

    $Order->get('LineItemContainer')->recalculate();

    my $PaymentTaxRate = $Order->get('LineItemContainer')->get('Payment')->get('TaxRate');
    is($PaymentTaxRate, 0.19, "$subTestName - Klarna TaxRate is not changed in LineItemPayment");

    my $PaymentLineItemPrice = $Order->get('LineItemContainer')->get('Payment')->get('LineItemPrice');
    is($PaymentLineItemPrice, 10, "$subTestName - Klarna LineItemPrice is recalculated and changed in LineItemPayment");

}

#----------------------------------------------------------------------------------------
sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

#----------------------------------------------------------------------------------------
sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'KlarnaMockShop.xml', $ROOTPATH );
    return 1;
}
RunScript( Sub => \&Main );

1;