#========================================================================================
# package      DE_EPAGES::MRW::API::Object::RemoteMRWBarcode
# state        private
#----------------------------------------------------------------------------------------
# description  This package provides the MRW Barcode SOAP Connection
#========================================================================================
package DE_EPAGES::MRW::API::RemoteMRWBarcode;
use base DE_EPAGES::MRW::API::RemoteMRW;
use strict;

use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::Core::API::PerlTools qw ( GetLocalTimeZone isValid );
use DE_EPAGES::Database::API::Connection qw ( GetCurrentDBHandle );
use DE_EPAGES::ExternalShipping::API::Log qw ( LogShippingFulfillment );
use DE_EPAGES::WebService::API::Client;
use Math::Round;

sub processShipmentRequest {
    my $self = shift;
    my ($ShippingFulfillmentMethod,$ShippingLabel,$FileType,$LanguageID) = @_;
    my $Shop = $ShippingFulfillmentMethod->getSite();
    my $hInfo = $ShippingFulfillmentMethod->get([qw( ShippingFulfillmentType WebserviceLocationCode WebserviceAccountNumber
                                            WebserviceDepartmentCode WebserviceUsername WebservicePassword)]);

    LogShippingFulfillment('MRW','ShippingFulfillmentMethod',$ShippingFulfillmentMethod,'TransmEnvio');
    LogShippingFulfillment('MRW','ShippingLabel',$ShippingLabel,'TransmEnvio');
    LogShippingFulfillment('MRW','LanguageID',$LanguageID,'TransmEnvio');

    my $isTestShop = $ShippingFulfillmentMethod->getSite()->get('IsInternalTestShop');
    my $WebserviceURL = $isTestShop ?
                $hInfo->{ShippingFulfillmentType}->get('WebserviceTestURL') :
                $hInfo->{ShippingFulfillmentType}->get('WebserviceURL') ;
   my $LocationCode = $hInfo->{WebserviceLocationCode};
   my $AccountNumber = $hInfo->{WebserviceAccountNumber};
   my $departmentCode = $hInfo->{WebserviceDepartmentCode};
   my $UserName = $hInfo->{WebserviceUsername};
   my $Password = $hInfo->{WebservicePassword};

    my $soap = $self->_getSoapClient( $WebserviceURL,
                                        $isTestShop);

    # get shop address
    my $ShopAddr = $ShippingLabel->getSite()->get('Address');

    #get the recipient address
    my $ShopRec = $ShippingLabel->get('Address');

    #Round the weight to the nearest integer
    my $roundedWeight = round($ShippingLabel->get('TotalWeight'));

    my $Formater = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    my $dt = GetCurrentDBHandle()->currentDateTime();
    # convert from UTC to local time
    $dt->set_time_zone( GetLocalTimeZone());

    my $FranchisingCode = SOAP::Data -> name('mrw:CodigoFranquicia')->value($LocationCode);
    my $AccountNr = SOAP::Data -> name('mrw:CodigoAbonado')->value($AccountNumber);
    my $DepartmentCode = SOAP::Data -> name('mrw:CodigoDepartamento')->value($departmentCode);
    my $Username = SOAP::Data -> name('mrw:UserName')->value($UserName);
    my $Pwd = SOAP::Data -> name('mrw:Password')->value($Password);


    my $AuthInfo = SOAP::Data
            ->name('mrw:AuthInfo' =>
                  \SOAP::Data->value(
                   $FranchisingCode,
                   $AccountNr,
                   $DepartmentCode,
                   $Username,
                   $Pwd
                   ));

    my $AddressPickupStreet2 = $ShopAddr->get('Street2');

    my $AddressPickup = SOAP::Data
            ->name('mrw:Direccion' =>
                  \SOAP::Data->value(
                   SOAP::Data -> name('CodigoTipoVia')->value(''),
                   SOAP::Data -> name('Via')->value($ShopAddr->get('Street')),
                   SOAP::Data -> name('Numero')->value(''),
                   SOAP::Data -> name('mrw:Resto')->value(isValid($AddressPickupStreet2) ? $AddressPickupStreet2 : ''),
                   SOAP::Data -> name('CodigoPostal')->value($ShopAddr->get('Zipcode')),
                   SOAP::Data -> name('Poblacion')->value($ShopAddr->get('City')),
                   SOAP::Data -> name('Provincia')->value($ShopAddr->get('State')),
                   SOAP::Data -> name('CodigoPais')->value(defined( $ShopAddr->get('Country') )
                                                                ? $ShopAddr->get('Country')->{'Code2'}
                                                                : undef)
                   ));

    my $AddressDeliveryStreet2 = $ShopRec->get('Street2');

    my $AddressDelivery = SOAP::Data
            ->name('mrw:Direccion' =>
                  \SOAP::Data->value(
                   SOAP::Data -> name('mrw:CodigoDireccion')->value(''),
                   SOAP::Data -> name('mrw:CodigoTipoVia')->value(''),
                   SOAP::Data -> name('mrw:Via')->value($ShopRec->get('Street')),
                   SOAP::Data -> name('mrw:Numero')->value(''),
                   SOAP::Data -> name('mrw:Resto')->value(isValid($AddressDeliveryStreet2) ? $AddressDeliveryStreet2 : ''),
                   SOAP::Data -> name('mrw:CodigoPostal')->value($ShopRec->get('Zipcode')),
                   SOAP::Data -> name('mrw:Poblacion')->value($ShopRec->get('City')),
                   SOAP::Data -> name('mrw:Provincia')->value($ShopRec->get('State')),
                   SOAP::Data -> name('mrw:CodigoPais')->value(defined( $ShopRec->get('Country') )
                                                                ? $ShopRec->get('Country')->{'Code2'}
                                                                : undef))
                   );

    my $MyPhone;
    my $ServiceData;
    my $NotificationRequest;
    my $Notifications;

    if (defined($ShopRec->get('PhoneCell'))){
        $MyPhone = $ShopRec->get('PhoneCell');
    }elsif (defined($ShopRec->get('Phone'))){
        $MyPhone = $ShopRec->get('Phone');
    }elsif (defined($ShopRec->get('PhonePrivate'))){
        $MyPhone = $ShopRec->get('PhonePrivate');
    }elsif (defined($ShopRec->get('PhoneBusiness'))){
        $MyPhone = $ShopRec->get('PhoneBusiness');
    }

    if (defined($MyPhone)){
        $NotificationRequest = SOAP::Data
                ->name('mrw:NotificacionRequest' =>
                      \SOAP::Data ->value(
                      SOAP::Data -> name('mrw:CanalNotificacion')->value('2'),
                      SOAP::Data -> name('mrw:TipoNotificacion')->value('2'),
                      SOAP::Data -> name('mrw:MailSMS')->value($MyPhone)
                      ));

        $Notifications = SOAP::Data
                ->name('mrw:Notificaciones' =>
                      \SOAP::Data ->value($NotificationRequest));

        $ServiceData = SOAP::Data
            ->name('mrw:DatosServicio' =>
                  \SOAP::Data->value(
                   SOAP::Data -> name('mrw:Fecha')->value($Formater->format_datetime($dt,'datetime', '%Y-%m-%dT%H:%M:%S')),
                   SOAP::Data -> name('mrw:CodigoServicio')->value($ShippingLabel->get('ShippingFulfillmentProduct')->get('Alias')),
                   SOAP::Data -> name('mrw:NumeroBultos')->value('1'),
                   SOAP::Data -> name('mrw:Peso')->value($roundedWeight),
                   SOAP::Data -> name('mrw:Notificaciones')->value($Notifications)
                   ));
    }else{
        $ServiceData = SOAP::Data
            ->name('mrw:DatosServicio' =>
                  \SOAP::Data->value(
                   SOAP::Data -> name('mrw:Fecha')->value($Formater->format_datetime($dt,'datetime', '%Y-%m-%dT%H:%M:%S')),
                   SOAP::Data -> name('mrw:CodigoServicio')->value($ShippingLabel->get('ShippingFulfillmentProduct')->get('Alias')),
                   SOAP::Data -> name('mrw:NumeroBultos')->value('1'),
                   SOAP::Data -> name('mrw:Peso')->value($roundedWeight)
                   ));
    }

    my $PickupData = SOAP::Data
            ->name('mrw:DatosRecogida' =>
                  \SOAP::Data->value(
                   $AddressPickup,
                   SOAP::Data -> name('mrw:Nombre')->value($ShopAddr->get('FullName')),
                   ));

    my $Company = $ShopRec->get('Company');
    my $FullName = $ShopRec->get('FullName');

    my $DeliveryData = SOAP::Data
            ->name('mrw:DatosEntrega' =>
                  \SOAP::Data->value(
                   $AddressDelivery,
                   SOAP::Data -> name('mrw:Nombre')->value(isValid($Company) ? $Company : $FullName),
                   SOAP::Data -> name('mrw:ALaAtencionDe')->value(isValid($Company) ? $FullName : ''),
                   SOAP::Data -> name('mrw:Telefono')->value(isValid( $ShopRec->get('Phone') )
                                                                ? $ShopRec->get('Phone')
                                                                : $ShopRec->get('PhoneCell')),
                   SOAP::Data -> name('mrw:Observaciones')->value($ShippingLabel->get('Order')->get('CustomerComment'))
                   ));

    my $Request = SOAP::Data
            ->name('mrw:request' =>
                  \SOAP::Data->value(
                   $PickupData,
                   $DeliveryData,
                   $ServiceData
                   ));

    my $header = SOAP::Header-> name('AuthInfo')->value($AuthInfo);
    my $body = SOAP::Data-> name('mrw:TransmEnvio')->value($Request);

    my $rSub = sub {
        my $ret;
        my $ErrorCode;
        my $ErrorMsg;

        eval {
          $ret = $soap->call('mrw:TransmEnvio', $header, $body);
        };
        if ( ExistsError() ) {
            my $Error = GetError();
            $ErrorCode = $Error->code;
            $ErrorMsg = $Error->shortMessage;
        }
        if ( ! defined $ret ) {
            $ErrorCode = -1;
            $ErrorMsg = 'MRW call has no response';
        }
        if ( defined $ErrorCode || defined $ErrorMsg ) {
            LogShippingFulfillment('MRW','webservice call failed for shop: '.$Shop->pathString,{
                'errorcode'    => $ErrorCode,
                'errormessage' => $ErrorMsg,
            },'TransmEnvio');

            if ( $ErrorCode && $ErrorCode eq 'WebServiceFault' && $ErrorMsg =~ /E2010/ ){
                $ErrorMsg = '401 Authorization Required';
            }
            Error('SOAP_REQUEST_FAILED', {'Code' => $ErrorCode, 'Message' => $ErrorMsg});
        }
        return $ret;
    };

    my $ret;
    if ( $self->doLog('MRW') ) {
        $ret =
          $soap->logCommunication( 'MRW::TransmEnvio',
            $rSub );
    }
    else {
        $ret = &$rSub;
    }
    my $requestStatus = $ret->dataof('//TransmEnvioResponse/TransmEnvioResult/Estado')->value;
    my $responseMessage = $ret->dataof('//TransmEnvioResponse/TransmEnvioResult/Mensaje')->value;
    my $TrackingNumber = $ret->dataof('//TransmEnvioResponse/TransmEnvioResult/NumeroEnvio')->value;
    #Create a hash with these values
    my $hResults = {};
    # if $requestStatus, no errors from MRW web service
    if ($requestStatus)
    {
        my $RequestNumber = $ret->dataof('//TransmEnvioResponse/TransmEnvioResult/NumeroSolicitud')->value;
        my $URL = $ret->dataof('//TransmEnvioResponse/TransmEnvioResult/Url')->value;
        my $label = $URL.'?Franq='.$LocationCode.'&Ab='.$AccountNumber.'&Dep=&Pwd='.$Password.'&NumSol='.$RequestNumber.'&Us='.$UserName.'&NumEnv='.$TrackingNumber;

        $hResults = {
            'Label' => $label,
            'TrackingNumber'=> $TrackingNumber,
            'requestStatus' => int ($requestStatus),
            'BarCodeError' => $responseMessage,
        };
    }
    else{
    $hResults = {
            'BarCodeError' => $responseMessage,
        };
    }
    return $hResults;

}
sub _getSoapClient {
    my $self = shift;
    my ( $WsURL, $testMode ) = @_;

    #Create the SOAP Object
    my $soap = DE_EPAGES::WebService::API::Client
                          ->proxy($WsURL)
                           ->on_action( sub { join '/', 'http://www.mrw.es', 'TransmEnvio' } ) # this is done to fix the SOAPAction header
                          ->encodingStyle('')
                          ->autotype(0)
                          ->soapversion('1.1')
                          ->readable(1);
    my $serializer = $soap->serializer();
    $serializer->register_ns( 'http://www.mrw.es/', 'mrw' );
    return $soap;
}
1;