#========================================================================================
# package      DE_EPAGES::Order::API::Attributes::Shop
# state        private
# description  Attribute interface for Shop.
#========================================================================================
package DE_EPAGES::Order::API::Attributes::Shop;
use base qw( DE_EPAGES::Object::API::BaseAttribute );

use strict;
use feature 'switch';
use DE_EPAGES::Object::API::Factory qw ( LoadClassByAlias );
use DE_EPAGES::Order::API::Table::MinOrderAmount qw (GetInfoMinOrderAmountsByShopID ExistsMinOrderAmount UpdateMinOrderAmount DeleteMinOrderAmount InsertMinOrderAmount);

sub getAttributes {
    shift;
    my ($Object, $aNames) = @_;
    my %Info;
    foreach my $Name (@$aNames) {
        given ($Name) {
            when('Orders') {
                $Info{$Name} = LoadClassByAlias('CustomerOrder')->getObjects('ByShopID', [$Object->id]);
            } 
            when('OrdersCount') {
                my $aObjects = LoadClassByAlias('CustomerOrder')->findObjects('ByShopID', [$Object->id]);
                $Info{$Name} = scalar(@$aObjects);
            }
            when('MinOrderAmounts') {
                my $ShopCurrencies = $Object->get('Currencies');
                my $aAmounts = GetInfoMinOrderAmountsByShopID($Object->id);
                my %Amounts;
                $Amounts{$_->{'CurrencyID'}} = $_->{'Value'} foreach @$aAmounts;
                $_->{'Value'} = $Amounts{$_->{'CurrencyID'}} foreach @$ShopCurrencies;
                $Info{$Name} = $ShopCurrencies;
            }
            when('AddToBasketAction') {
                $Info{$Name} = $Object->get('AddProductStyle');
            }
            default {
                die "Invalid attribute: $Name";
            }
        }
    }
    return \%Info;
}
sub setAttributes {
    shift;
    my ($Object, $hValues) = @_;
    my $ShopID = $Object->id;
    foreach my $Name (keys %$hValues) {
        if ($Name eq 'MinOrderAmounts') {
            foreach my $rEntry (@{$hValues->{$Name}}) {
                my $Entry = {
                    %$rEntry,
                    'ShopID' => $ShopID,
                };
                if (ExistsMinOrderAmount($Entry)) {
                    if (defined $Entry->{'Value'}) {
                        UpdateMinOrderAmount($Entry);
                    } else {
                        DeleteMinOrderAmount($Entry);
                    }
                } elsif (defined $Entry->{'Value'}) {
                    InsertMinOrderAmount($Entry);
                }
            }
        }
    }
}

sub deleteObject {
    shift;
    my ($Object) = @_;

    LoadClassByAlias('CustomerOrder')->deleteObjects('ByShopID', [$Object->id]);
    LoadClassByAlias('Basket')->deleteObjects('ByShopID', [$Object->id]);
    DE_EPAGES::Order::API::Table::MinOrderAmount::OnDeleteShop({'ShopID'=>$Object->id});
}


sub exportXMLElements {
    my $self = shift;
    my ($Driver, $Object, $aNames, $hAttributes) = @_;

    if ($Driver->exportLevel == $Object->class->attribute('MinOrderAmounts')->get('ExportLevel')) {
        foreach ( grep { defined $_->{'Value'} } @{$Object->getSite->get('MinOrderAmounts')} ) {
            $Driver->writeElement('MinOrderAmount', {
                'CurrencyID' => $_->{'CurrencyID'},
                'Value' => $_->{'Value'},
            } );
        }
    }
}

1;
