package DE_EPAGES::Product::t::ProductResource;
use base DE_EPAGES::Shop::API::Test;

use strict;
use Test::More tests => 3;

use Data::Dumper;
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw( ImportObjects );
use DE_EPAGES::Product::API::Resource::Product;

## no critic qw(ProhibitMagicNumbers)

sub test {
    my $self = shift;

    $self->test_get;
    $self->test_get_master;
    $self->test_get_sub;
}

sub test_get {
    my $self = shift;

    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';

    my $Resource = DE_EPAGES::Product::API::Resource::Product->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyPetrol');
    my $SiteLocale = LoadRootObject->siteLocale($LocaleID);
    my $TaxType = 'GROSS';
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $SfBaseUrl = $self->shop->get('WebUrl') ;

    my $aCategories = $Product->get('AllProductCategories');
    my $MainCategory = $aCategories->[0];

    my $hData = $Resource->get( $Product, $LocaleID, $CurrencyID,  );

    # remove guid of tax class from result since it is different each time.
    delete $hData->{'priceInfo'}->{'taxClass'}->{'taxClassId'};

    is_deeply( $hData, {
        productWidth=>'100',
        productLength=>'120',
        productHeight=>'110',
        minStocklevel=>'3',
        manufacturerProductNumber=>'Patrol99',
        visible => JSON::true,
        availability => 'OnStock',
        availabilityText => '',
        deliveryWeight => undef,
        deliveryPeriod => undef,
        description => undef,
        title => 'MyPetrol',
        forSale => 'true',
        productImage => undef,
        images => [
        ],
        specialOffer => 'false',
        name => 'MyPetrol',
        productVariationType => 'regular',
        productVariationMasterName => undef,
        productVariationMasterId => undef,
        productVariationSelection => undef,
        priceInfo => {
            basePrice => undef,
            depositPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            ecoParticipationPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            manufacturerPrice => _Price( $SiteLocale, 0.6, $CurrencyID, $TaxType ),
            price => _Price( $SiteLocale, 1.19, $CurrencyID, $TaxType ),
            priceWithDeposits => _Price( $SiteLocale, 1.19, $CurrencyID, $TaxType ),
            lowestPrice => undef,
            highestPrice => undef,
            quantity => {
                amount => 1,
                unit => 'l'
            },
            'taxClass' => {
                'name' => 'standard',
                'percentage' => '19'
            },
        },
        productId => $Product->guid,
        shortDescription => undef,
        shippingMethodsRestrictedTo => undef,
        sfUrl => $SfBaseUrl.'/?ObjectPath=/Test/TestShop/Products/MyPetrol',
        videos => [
            {
                'source' => '08154711',
                'type' => 'YouTube'
            }
        ],
        links => [
            {
                rel => 'self',
                href => $SSLDomain.'/api/products/'.$Product->guid,
            },
            {
                rel => 'categories',
                href => $SSLDomain.'/api/categories/?productId='.$Product->guid,
            },
            {
                rel => 'slideshow',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/slideshow',
            },
            {
              'rel' => 'main-category',
              'href' => $SSLDomain.'/api/categories/'.$MainCategory->guid(),
              'title' => 'min en'
            }
        ],
        energyLabelsString => undef,
        stocklevel => '500',
        energyLabelSourceFile => undef,
        energyLabelsMaxEfficiencyString => undef,
        productDataSheet => undef,
        productNumber => 'MyPetrol',
        manufacturer => undef,
        upcEan => undef,
        essentialFeatures => 'test product petrol',
        searchKeywords => ['Product', 'Search', 'Keywords'],
    }, 'GET MyPetrol' ) or Test::More::diag Dumper($hData);
}

sub test_get_master {
    my $self = shift;

    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';

    my $Resource = DE_EPAGES::Product::API::Resource::Product->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket');
    my $SiteLocale = LoadRootObject->siteLocale($LocaleID);
    my $TaxType = 'GROSS';
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $SfBaseUrl = $self->shop->get('WebUrl') ;

    my $aCategories = $Product->get('AllProductCategories');
    my $MainCategory = $aCategories->[0];

    my $hData = $Resource->get( $Product, $LocaleID, $CurrencyID );

    # remove guid of tax class from result since it is different each time.
    delete $hData->{'priceInfo'}->{'taxClass'}->{'taxClassId'};

    is_deeply( $hData, {
        productWidth=>undef,
        productLength=>undef,
        productHeight=>undef,
        minStocklevel=>undef,
        manufacturerProductNumber=>undef,
        visible => JSON::true,
        availability => undef,
        availabilityText => '',
        forSale => 'false',
        deliveryWeight => undef,
        deliveryPeriod => undef,
        description => undef,
        title => 'MyJacket',
        shortDescription => undef,
        productImage => undef,
        images => [{
           'url' => $SSLDomain.$Product->get('WebPath').'/myjacket.png',
           'classifier' => 'Small',
           'isZoomEnabled' => 'false',
           'width' => undef,
           'height' => undef,
           'fileSize' => undef
        }],
        specialOffer => 'false',
        name => 'MyJacket',
        productVariationType => 'master',
        productVariationMasterName => undef,
        productVariationMasterId => undef,
        productVariationSelection => undef,
        priceInfo => {
            basePrice => undef,
            depositPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            ecoParticipationPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            manufacturerPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            price => _Price( $SiteLocale, 199.95, $CurrencyID, $TaxType ),
            priceWithDeposits => _Price( $SiteLocale, 199.95, $CurrencyID, $TaxType ),
            lowestPrice => _Price( $SiteLocale, 189.95, $CurrencyID, $TaxType),
            highestPrice => _Price( $SiteLocale, 199.95, $CurrencyID, $TaxType),
            quantity => {
                amount => 1,
                unit => 'piece(s)'
            },
            'taxClass' => {
                'name' => 'standard',
                'percentage' => '19'
            }
        },
        stocklevel => undef,
        productId => $Product->guid,
        shortDescription => undef,
        shippingMethodsRestrictedTo => undef,
        sfUrl => $SfBaseUrl.'/?ObjectPath=/Test/TestShop/Products/MyJacket',
        videos => [],
        links => [
            {
                rel => 'self',
                href => $SSLDomain.'/api/products/'.$Product->guid,
            },
            {
                rel => 'variations',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/variations',
            },
            {
                rel => 'lowest-price',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/lowest-price',
            },
            {
                rel => 'categories',
                href => $SSLDomain.'/api/categories/?productId='.$Product->guid,
            },
            {
                rel => 'custom-attributes',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/custom-attributes',
            },
            {
              'rel' => 'main-category',
              'href' => $SSLDomain.'/api/categories/'.$MainCategory->guid(),
              'title' => 'min en'
            }
        ],
        energyLabelsString => undef,
        energyLabelsMaxEfficiencyString => undef,
        energyLabelSourceFile => undef,
        productDataSheet => undef,
        productNumber => 'MyJacket',
        manufacturer => undef,
        upcEan => undef,
        essentialFeatures => 'test product jacket',
        searchKeywords => ['Product', 'Search', 'Keywords', 'Jacket'],
    }, 'GET MyJacket' ) or Test::More::diag Dumper($hData);
}

sub test_get_sub {
    my $self = shift;

    my $LocaleID = 'en_GB';
    my $CurrencyID = 'EUR';

    my $Resource = DE_EPAGES::Product::API::Resource::Product->new;

    my $ProductFolder = $self->shop->get('ProductFolder');
    my $Product = $ProductFolder->child('MyJacket')->child('SubProducts')->child('JacketBlack2');
    my $SiteLocale = LoadRootObject->siteLocale($LocaleID);
    my $TaxType = 'GROSS';
    my $SSLDomain = $self->shop->get('ProtocolAndServerSSL');
    my $SfBaseUrl = $self->shop->get('WebUrl') ;

    my $aCategories = $Product->get('AllProductCategories');
    my $MainCategory = $aCategories->[0];

    my $hData = $Resource->get( $Product, $LocaleID, $CurrencyID,  );

    # remove guid of tax class from result since it is different each time.
    delete $hData->{'priceInfo'}->{'taxClass'}->{'taxClassId'};

    is_deeply( $hData, {
        productWidth=>undef,
        productLength=>undef,
        productHeight=>undef,
        minStocklevel=>undef,
        manufacturerProductNumber=>undef,
        visible => JSON::true,
        availability => 'OnStock',
        availabilityText => '',
        forSale => 'true',
        deliveryWeight => undef,
        deliveryPeriod => undef,
        description => undef,
        title => 'JacketBlack2',
        productImage => undef,
        images => [{
           'url' => $SSLDomain.$Product->get('WebPath').'/myjacket.png',
           'classifier' => 'Small',
           'isZoomEnabled' => 'false',
           'width' => undef,
           'height' => undef,
           'fileSize' => undef
        }],
        specialOffer => 'false',
        name => 'JacketBlack2',
        productVariationType => 'variation',
        productVariationMasterName => 'MyJacket',
        productVariationMasterId => '5CAB2F17-F672-819F-F7D9-AC1100051604',
        productVariationSelection => [
         {
           'name' => 'Size',
           'displayName' => 'Size',
           'value' => 'M',
           'displayValue' => 'M'
         },
         {
           'name' => 'Color',
           'displayName' => 'Color',
           'value' => 'Black',
           'displayValue' => 'Black'
         }
        ],
        priceInfo => {
            basePrice => undef,
            depositPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            ecoParticipationPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            manufacturerPrice => _Price( $SiteLocale, undef, $CurrencyID, $TaxType ),
            price => _Price( $SiteLocale, 189.95, $CurrencyID, $TaxType ),
            priceWithDeposits => _Price( $SiteLocale, 189.95, $CurrencyID, $TaxType ),
            lowestPrice => undef,
            highestPrice => undef,
            quantity => {
                amount => 1,
                unit => 'piece(s)'
            },
            'taxClass' => {
                'name' => 'standard',
                'percentage' => '19'
            }
        },
        productId => $Product->guid,
        shortDescription => undef,
        stocklevel => '4',
        shippingMethodsRestrictedTo => undef,
        sfUrl => $SfBaseUrl.'/?ObjectPath=/Test/TestShop/Products/MyJacket/SubProducts/JacketBlack2',
        videos => [],
        links => [
            {
                rel => 'self',
                href => $SSLDomain.'/api/products/'.$Product->guid,
            },
            {
                rel => 'variations',
                href => $SSLDomain.'/api/products/'.$Product->superProduct->guid.'/variations',
            },
            {
                rel => 'master',
                href => $SSLDomain.'/api/products/'.$Product->superProduct->guid,
            },
            {
                rel => 'categories',
                href => $SSLDomain.'/api/categories/?productId='.$Product->guid,
            },
            {
                rel => 'custom-attributes',
                href => $SSLDomain.'/api/products/'.$Product->guid.'/custom-attributes',
            },
            {
              'rel' => 'main-category',
              'href' => $SSLDomain.'/api/categories/'.$MainCategory->guid(),
              'title' => 'min en'
            }
        ],
        energyLabelsString => undef,
        energyLabelsMaxEfficiencyString => undef,
        energyLabelSourceFile => undef,
        productDataSheet => undef,
        productNumber => 'JacketBlack2',
        manufacturer => undef,
        upcEan => undef,
        essentialFeatures => 'test product jacket 2',
        searchKeywords => ['Product', 'Search', 'Keywords', 'Jacket'],
    }, 'GET JacketBlack2' ) or Test::More::diag Dumper($hData);
}

sub _Price {
    my ($SiteLocale, $Amount, $CurrencyID, $TaxType) = @_;

    return undef unless defined $Amount;

    return {
        amount => $Amount,
        currency => $CurrencyID,
        formatted => defined $Amount ? $SiteLocale->formatMoney($Amount, $CurrencyID) : undef,
        taxType => $TaxType,
    };
}

sub init {
    my $self = shift;

    $self->SUPER::init;
    die 'Shop alias is not TestShop' unless $self->shop->alias eq 'TestShop';
    $self->shop->set({
        DomainName => 'test-domain',
        HasSSLCertificate => 1
    });

    note 'importing shop xml';
    ImportObjects('ShopXMLs/Shop_for_ProductsResource_test.xml', $self->shop->parent->pathString);
}

__PACKAGE__->runScript;
