#========================================================================================
# package      DE_EPAGES::Shop::API::Attributes::ShopNotification_Ext
# state        private
# description  additional attributes for ShopNotifications
#========================================================================================
package DE_EPAGES::Shop::API::Attributes::ShopNotification_Ext;
use base DE_EPAGES::Object::API::BaseAttribute;

use strict;
use feature 'switch';
use DE_EPAGES::Core::API::CSV qw ( ParseCSV );
use DE_EPAGES::Object::API::Factory qw ( LoadObject ExistsObject );

#========================================================================================
# function     getAttributes
# state        private
#----------------------------------------------------------------------------------------
# syntax       $hValues = $Package->( $Object, $aNames );
# example      # never call getAttributes directly!
#               $Value = $Object->get( $AttributeName );
#----------------------------------------------------------------------------------------
# description  Returns attribute values of an object.
#               Used by <function DE_EPAGES::Object::Object::Object::get>
#----------------------------------------------------------------------------------------
# input        $Object | object | object
# input        $aNames | list of attribute names | ref.array
# return       $hValues | attribute name => value hash | ref.hash
#========================================================================================
sub getAttributes {
    my $self = shift;
    my ($Object, $aNames, $LanguageID) = @_;

    my %Values;
    foreach my $Name (@$aNames) {
        given ($Name) {
            when('ShopNotificationObjects'){
                my $aParsed = [];
                my $String = $Object->get('ShopNotificationObjectIDString');
                $aParsed = ParseCSV($Object->get('ShopNotificationObjectIDString')) if (defined $String && $String =~ /\S/);

                my @ShopNotificationObjects;
                foreach my $ObjectID (@$aParsed) {
                    if (ExistsObject($ObjectID)) {
                        # only retrun objects that actually exists
                        push @ShopNotificationObjects, LoadObject($ObjectID);
                    } else {
                        # remove non-existing objects from the notification
                        $Object->deleteObjectID($ObjectID);
                    }
                }
                $Values{$Name} = \@ShopNotificationObjects;
            }
            when('ShopNotificationObjectsCount'){
                my $aParsed = [];
                my $String = $Object->get('ShopNotificationObjectIDString');
                $aParsed = ParseCSV($Object->get('ShopNotificationObjectIDString')) if (defined $String && $String =~ /\S/);
                $Values{$Name} = scalar(@$aParsed);
            }
            default {
                die "Invalid attribute: $Name";
            }
        }
    }

    return \%Values;
}

1;
